package ch6;

import java.awt.*;
import java.awt.image.*;
import javax.swing.*;
import javax.media.jai.*;
import javax.media.jai.widget.*;

public class TileTester extends JFrame implements TileObserver {

    /**
       TileTester.java - takes two images of the same size and uses
       tiles from the first image to edit tiles in the second image.  
    */
    public TileTester(String filename1, String filename2) {
	RenderedOp inputRO1 = JAI.create("fileload", filename1);
	RenderedOp inputRO2 = JAI.create("fileload", filename2);
	if ( (inputRO1.getWidth() != inputRO2.getWidth()) ||
	     (inputRO1.getHeight() != inputRO2.getHeight()) ) {
	    System.err.print("Images must have same dimensions ");
	    System.err.println("for this example to run properly");
	    System.exit(1);
	}

	/* 
	   Create two TiledImages, one for each RenderedOp.

	   We are specifying the tile size as half the 
	   width and height of the source RenderedOps

	   Thus, each TiledImage will have 4 tiles, 
	   tile(0,0), tile(0,1), tile(1,0) and tile(1,1);
	*/	   
	TiledImage ti1 = new TiledImage(inputRO1, 
					inputRO1.getWidth()/2, 
					inputRO1.getHeight()/2);
	TiledImage ti2 = new TiledImage(inputRO2, 
					inputRO2.getWidth()/2, 
					inputRO2.getHeight()/2);

	//addTileObserver for the 2nd TiledImage
	ti2.addTileObserver(this);

	//ti2copy will copy data from ti2's DataBuffer
	TiledImage ti2copy = new TiledImage(ti2, false); 

	//ti2share will share ti2's DataBuffer
	TiledImage ti2share = new TiledImage(ti2, true);  

	/* 
	   Force rendering of ti2copy and ti2share.
	   Rendering either will cause ti2 to be rendered.
	   Displaying them will also cause them to be rendered,
	   but it happens in a separate thread.  This way we
	   have more control.
	*/
	Raster[] tmpR;
	tmpR = ti2copy.getTiles(); // render ti2copy
	tmpR = ti2share.getTiles(); // render ti2share

	// now display the TiledImage
	getContentPane().setLayout(new GridLayout(2,2));
	getContentPane().add(new ch6Display(ti1));
	getContentPane().add(new ch6Display(ti2));
	getContentPane().add(new ch6Display(ti2copy));
	getContentPane().add(new ch6Display(ti2share));

	pack();
	show();

	/*
	  take tile(0,0) from TiledImage ti1 and use it to replace
	  tile(0,0) in TiledImage ti2.  

	  This will only effect ti2 and ti2share, not ti2copy.  

	  Also, the setData method will cause ti2 to generate two tile
	  update events.  One for when tile with index 0,0 is about to
	  become writable and one when it is about to be released.

	  Both of which happen implicitly since there are no calls to
	  getWritableTile or releaseWritableTile 
	*/

	Raster r00 = ti1.getTile(0,0);
	ti2.setData(r00); 
	repaint();

	/*
	  copy tile(1,1) from TiledImage t1 and use it to 
	  replace tile(1,1) in ti2.
	  Again ti2 generates two tile update events.  
	  Both of these happen explicitly;
	  one when getWritableTile is called and one 
	  when releaseWritableTile is called
	*/
	Raster ri11 = ti1.getTile(1,1);
	WritableRaster wr = ti2.getWritableTile(1,1);
	wr.setRect(0,0,ri11); 
	ti2.releaseWritableTile(1,1);
	repaint();
    }

    /*
      this method gets called to handle any tile update events
    */
    public void tileUpdate(WritableRenderedImage source,
			   int tileX,
			   int tileY,
			   boolean willBeWritable) {
	System.out.println("Tile("+tileX+","+tileY+")");
	if (willBeWritable) 
	    System.out.println(" is writable");
	else 
	    System.out.println(" is not writable");
    }

    /**
       input should be two filenames representing equal sized images
    */
    static public void main(String[] args) {
	if (args.length != 2)
	    System.err.println("Usage: TileTester filename1 filename2 ");
	else
	    new TileTester(args[0], args[1]);
    }
}
